
;
; written by ville helin <vhelin@iki.fi> in 2004
;

.include "defines.i"

;
; c4x4's variables in high ram
;

.define C4x4_LINECO ($80+C4x4_COPY_END-C4x4_COPY+0)
.define C4x4_FRAMEP ($80+C4x4_COPY_END-C4x4_COPY+1)
.define C4x4_COPY_RAM $ff80
.export C4x4_LINECO, C4x4_FRAMEP

;
; c4x4 - chunky mode (80x72) with 4 unique colors each scanline
;

.bank 0 slot 0
.org $150
.section "c4x4" semifree

;
; c4x4 init - build the 4x1 tiles
;

C4x4_INIT:
	ld	b, 128
	ld	hl, $8800

-	ld	a, b
	and	%01010101	; low bits
	ld	d, a
	sla	a
	or	d		; 4x1
	ld	d, a		; d ~ low 4x1

	ld	a, b
	and	%10101010	; high bits
	ld	e, a
	sra	a
	or	e		; 4x1
*	ld	e, a		; e ~ high 4x1

	.rept 8			; create the tile
	ld	(hl), d
	inc	hl
	ld	(hl+), a
	.endr

	inc	b
	ld	a, b
	cp	128
	jr	nz, -

	; touch the display settings...
	ld	hl, $ff40	; $8800+
	res	4, (hl)
	res	3, (hl)
	set	0, (hl)
	xor	a
	ld	hl, $ff42
	ld	(hl), a
	inc	hl
	ld	(hl), a

	; init the lcd irq
	ld	hl, IRQ_LCD
	ld	a, $C3		;jp.
	ld	(hl+), a
	ld	de, C4x4_SKIP
	ld	a, e
	ld	(hl+), a
	ld	(hl), d

	; copy the tile copier to ram
	ld	hl, C4x4_COPY_RAM
	ld	de, C4x4_COPY
	ld	b, C4x4_COPY_END-C4x4_COPY
	call	MEMCPY_BYTECOPY

	ret

;
; c4x4 tile copy - the code, which changes the tiles to quandruple the
; vertical resolution
;

C4x4_COPY:
	push	af
	push	hl

	; load the output address (to be modified)
	ld	hl, $9800

*	ld	a, ($ff00+R_LY)
*	cp	0
*	jr	z, +
*	.db	$e4
*+	.db	$e3

C4x4_COPY_TILES:
	; copy the 20 tiles
	.rept 19
	ld	a, $63		; the data here will be modified
	ld	(hl+), a
	.endr
	ld	(hl), $63	; the last one

*	ld      a, ($ff00+R_STAT)
*	and     %11
*	jr      z, +
*	.db $e4
*+
	; swap in the scanline skipper lcd irq
	ld	a, C4x4_SKIP>>8	; high
	ld	(IRQ_LCD+2), a

	; load the palette 0 of background
	ld	a, %10000000
	ld	($ff00+R_BCPS), a

	ld	a, ($ff00+C4x4_FRAMEP+0)
	ld	l, a
	ld	a, ($ff00+C4x4_FRAMEP+1)
	ld	h, a

	; load the eight bytes
	.rept 8
	ld	a, (hl+)
	ld	($ff00+R_BCPD), a
	.endr

	; save the pointer
	ld	a, l
	ld	($ff00+C4x4_FRAMEP+0), a
	ld	a, h
	ld	($ff00+C4x4_FRAMEP+1), a

	pop	hl
	pop	af

	reti
C4x4_COPY_END:

;
; c4x4 frame initializer
;

C4x4_LOAD_PALETTE:
	; the four colors follow every 20 bytes of scanline data


;
; c4x4 frame initializer
;

C4x4_INIT_FRAME:
	; reset the destination to $9800
	ld	hl, C4x4_COPY_RAM+3
	xor	a

	; init the line counter
	ld	($ff00+C4x4_LINECO), a
	
	ld	(hl+), a
	ld	(hl), $98

	; set the lcd interrupt
	ld	a, C4x4_SKIP>>8	; high
	ld	(IRQ_LCD+2), a

	ret

;
; c4x4 tile copier setup - copy tiles to the ram tile copy function
;

.macro C4x4_INIT_COPY_TILE
	; copy the next 20 tiles
	ld	a, ($ff00+C4x4_FRAMEP+0)
	ld	l, a
	ld	a, ($ff00+C4x4_FRAMEP+1)
	ld	h, a

	ld	a, (hl+)
	ld	($ff00+$80+6), a
	ld	a, (hl+)
	ld	($ff00+$80+9), a
	ld	a, (hl+)
	ld	($ff00+$80+12), a
	ld	a, (hl+)
	ld	($ff00+$80+15), a
	ld	a, (hl+)
	ld	($ff00+$80+18), a
	ld	a, (hl+)
	ld	($ff00+$80+21), a
	ld	a, (hl+)
	ld	($ff00+$80+24), a
	ld	a, (hl+)
	ld	($ff00+$80+27), a
	ld	a, (hl+)
	ld	($ff00+$80+30), a
	ld	a, (hl+)
	ld	($ff00+$80+33), a

	ld	a, (hl+)
	ld	($ff00+$80+36), a
	ld	a, (hl+)
	ld	($ff00+$80+39), a
	ld	a, (hl+)
	ld	($ff00+$80+42), a
	ld	a, (hl+)
	ld	($ff00+$80+45), a
	ld	a, (hl+)
	ld	($ff00+$80+48), a
	ld	a, (hl+)
	ld	($ff00+$80+51), a
	ld	a, (hl+)
	ld	($ff00+$80+54), a
	ld	a, (hl+)
	ld	($ff00+$80+57), a
	ld	a, (hl+)
	ld	($ff00+$80+60), a
	ld	a, (hl+)
	ld	($ff00+$80+63), a

	; save the pointer
	ld	a, l
	ld	($ff00+C4x4_FRAMEP+0), a
	ld	a, h
	ld	($ff00+C4x4_FRAMEP+1), a
.endm

;
; c4x4 vbi
;

C4x4_VBI:
	push	af
	push	hl
	push	bc

	; init the frame pointer
	ld	hl, DAT_LOGO1
	ld	a, l
	ld	($ff00+C4x4_FRAMEP+0), a
	ld	a, h
	ld	($ff00+C4x4_FRAMEP+1), a

	; reset the destination
	call	C4x4_INIT_FRAME

	; copy the first 20 tiles directly into vram
	ld	a, ($ff00+C4x4_FRAMEP+0)
	ld	l, a
	ld	a, ($ff00+C4x4_FRAMEP+1)
	ld	h, a
	ld	bc, $9800

	.rept 19
	ld	a, (hl+)
	ld	(bc), a
	inc	c		; inc c is enough as c starts from $00.
	.endr
	ld	a, (hl+)
	ld	(bc), a

	; the four colors follow every 20 bytes of scanline data
	ld	a, %10000000
	ld	($ff00+R_BCPS), a

	; load the eight bytes
	.rept 8
	ld	a, (hl+)
	ld	($ff00+R_BCPD), a
	.endr

	; save the pointer
	ld	a, l
	ld	($ff00+C4x4_FRAMEP+0), a
	ld	a, h
	ld	($ff00+C4x4_FRAMEP+1), a

	pop	bc
	pop	hl
	pop	af

	reti

.ends

;
; c4x4 scanline skipper interrupt (this needs to be in $XY80, becase the
; the scanline copier routine is in $ff80, and now we just need to change
; the high byte to change the irq).
;

.org $2080
.section "c4x4_skip" force

C4x4_SKIP:
	push	af
	push	hl

	; modify the tile copier for the next copy
	C4x4_INIT_COPY_TILE

	; next scanline we will copy the tiles
	ld	a, C4x4_COPY_RAM>>8	; high
	ld	(IRQ_LCD+2), a

	; next tile row?
	ld	a, ($ff00+C4x4_LINECO)
	inc	a
	ld	($ff00+C4x4_LINECO), a
	and	%11
	jr	z, +			; this jump will take place every fourth call

	; exit
	pop	hl
	pop	af

	reti

	; ... next tile row!
+	ld	a, ($ff00+$80+3)	; low
	add	32
	ld	($ff00+$80+3), a
	ld	a, ($ff00+$80+4)	; high
	adc	0
	ld	($ff00+$80+4), a

	pop	hl
	pop	af

	reti

.ends
